@@ -41,6 +41,7 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
     def __init__(self):
         self._depth = None
         self._id_scopes = {}
+        self._projection_stack = []
 
     def visit(self, node):
         if hasattr(node, 'rep'):
@@ -153,15 +154,18 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         return node
 
     def visit_Compare(self, node):
-        left_rep = self.get_rep(node.left)
-        node.rep = '(' + left_rep
-        for operator, comparator in zip(node.ops, node.comparators):
+        comparison_reps = []
+        full_comparators = [node.left] + node.comparators
+        for left, operator, right in zip(full_comparators[:-1], node.ops, full_comparators[1:]):
             if type(operator) not in compare_op_dict:
                 raise SyntaxError('Unimplemented comparison operation: ' + operator)
+            left_rep = self.get_rep(left)
             operator_rep = compare_op_dict[type(operator)]
-            comparator_rep = self.get_rep(comparator)
-            node.rep += ' ' + operator_rep + ' ' + comparator_rep
-        node.rep += ')'
+            right_rep = self.get_rep(right)
+            comparison_reps.append('(' + left_rep + ' ' + operator_rep + ' ' + right_rep + ')')
+        node.rep = ' & '.join(comparison_reps)
+        if len(node.ops) > 1:
+            node.rep = '(' + node.rep + ')'
         return node
 
     def visit_IfExp(self, node):
@@ -311,12 +315,24 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                             + len(node.selector.args.args))
         self.visit(node.source)
         self._depth += 1
-        call_node = ast.Call(func=node.selector, args=[node.source])
+        self._projection_stack.append(node.selector.args.args[0].arg)
+        if self._depth > 2:
+            rep1, rep2 = self._projection_stack[-2], self._projection_stack[-1]
+            lambda_node = ast.Lambda(args=ast.arguments(args=[ast.arg(arg=rep1),
+                                                              ast.arg(arg=rep2)]),
+                                     body=node.selector.body)
+            call_rep = (self.get_rep(lambda_node) + '(*ak.unzip(ak.cartesian(('
+                        + rep1 + ', ' + self.get_rep(node.source)
+                        + '), axis=' + repr(self._depth - 2) + ', nested=True)))')
+        else:
+            call_node = ast.Call(func=node.selector, args=[node.source])
+            call_rep = self.get_rep(call_node)
         node.rep = ('(lambda selection: ak.zip(selection,'
                     + ' depth_limit=(None if len(selection) == 1 else ' + repr(self._depth) + '))'
                     + ' if not isinstance(selection, ak.Array)'
-                    + ' else selection)(' + self.get_rep(call_node) + ')')
+                    + ' else selection)(' + call_rep + ')')
         self._depth -= 1
+        self._projection_stack.pop()
         return node
 
     def visit_SelectMany(self, node):
@@ -400,3 +416,16 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         node.rep = self.get_rep(call_node)
         self._depth -= 1
         return node
+
+    def visit_First(self, node):
+        node.rep = self.get_rep(node.source) + '[' + ':, ' * self._depth + '0]'
+        return node
+
+    def visit_ElementAt(self, node):
+        node.rep = (self.get_rep(node.source) + '[' + ':, ' * self._depth
+                    + self.get_rep(node.index) + ']')
+        return node
+
+    def visit_Last(self, node):
+        node.rep = self.get_rep(node.source) + '[' + ':, ' * self._depth + '-1]'
+        return node
