@@ -1,3 +1,5 @@
+import copy
+
 import qastle
 
 from .transformer import PythonSourceGeneratorTransformer
@@ -10,13 +12,15 @@ def python_ast_to_python_source(python_ast):
 
 def generate_python_source(ast, function_name='run_query'):
     if isinstance(ast, str):
-        ast = qastle.text_ast_to_python_ast(ast)
-    qastle.insert_linq_nodes(ast)
+        python_ast = qastle.text_ast_to_python_ast(ast)
+    else:
+        python_ast = copy.deepcopy(ast)
+    python_ast = qastle.insert_linq_nodes(python_ast)
     source = ('def ' + function_name
               + '(' + input_filenames_argument_name + '=None, '
               + tree_name_argument_name + '=None):\n')
-    source += '    import logging, numpy as np, awkward as ak, uproot\n'
-    source += '    return ' + python_ast_to_python_source(ast) + '\n'
+    source += '    import logging, numpy as np, awkward as ak, uproot, vector\n'
+    source += '    return ' + python_ast_to_python_source(python_ast) + '\n'
     return source
 
 
